==> provider_software_3.6.4.tar.gz: provider_software_3.6.4/lib/biocase/wrapper/dbmod/dbmod_odbc_tsql_2012+.py <==
# -*- coding: utf8 -*-
'''
DB MODULE - ODBC for TSQL
*******************************
***  DB MODULE - ODBC for TSQL
*******************************
$RCSfile: dbmod_odbc_tsql.py,v $
$Revision: 1302 $
$Author: j.holetschek $
$Date: 2013-07-08 14:14:06 +0200 (Mo, 08 Jul 2013) $

The biocase.wrapper database module to access MS SQL Server databases.
You will need the mxODBC module to connect to a MS SQL Server DB via ODBC.
'''

from biocase.wrapper.sql.base import sqlBase
from biocase.wrapper.dbmod.base_dbmod import DBBasemodule
from biocase.wrapper.errorclasses import DbModuleMissingError, ConnectionError

import logging
log = logging.getLogger("pywrapper.dbmod")


#############################################
#
# CUSTOMIZATION for SQL Server
#
# OVERLOAD BASE CLASSES AND OTHER METHODS
#
#############################################

# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'
sqlBase.templates['select-limit'] = 'SELECT DISTINCT TOP #3 #1 FROM #2'
sqlBase.templates['count-distinct'] = 'COUNT(DISTINCT #1)'

# Works only with SQL Server 2012 (or later)
sqlBase.templates['select-limit-offset'] = 'SELECT DISTINCT #1 FROM #2 OFFSET #4 ROWS FETCH NEXT #3 ROWS ONLY'

# ESCAPE CHARACTERS
# -----------------
# characters that need to be escaped in SQL
# default list contains single quotes to be escaped into two single quotes
# but that's already done b< ceODBC
sqlBase.escape = {}


class DBmodule(DBBasemodule):
    __version__ = 'ceODBC ?'
    __dbmod__ = u'MS SQL Server module'

    def __init__(self, psfObj):
        # setting the psfObj reference to self.psfObj
        # call createDBconnection method
        DBBasemodule.__init__(self, psfObj)
        # dont use parameters but integrate all parameters into SQL strings:
        self.trans.integrateParametersInSQLString()

    # ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
    def createDBconnection(self):
        try:
            import ceODBC as odbc
        except:
            raise DbModuleMissingError('ceODBC')

        self.__class__.__version__ = self.__class__.__version__[:-1] + odbc.version
        loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.database, self.psfObj.user, self.psfObj.password)
        try:
            return odbc.connect(loginParas)
        except:
            raise ConnectionError()

    def getTables(self):
        tables = []
        for t in self.conn.tables():
            if (str(t[3]) == 'TABLE' or str(t[3]) == 'VIEW') and not (t[1] in ["INFORMATION_SCHEMA", "sys"] or t[2] in ["dtproperties", "sysconstraints", "syssegments"]):
                tables.append(unicode(t[2]))
        return tables

    def getColumns(self, _table):
        try:
            ret = [unicode(c[3]) for c in self.conn.columns(catalog=None, schema=None, table=_table)]
            if ret == None:
                return []
            return ret
        except:
            log.error("error fetching columns")
            return []

    def hasMetaInfo(self):
        return True

==> provider_software_3.7.4.tar.gz: provider_software_3.7.4/lib/biocase/wrapper/dbmod/dbmod_odbc_tsql_2012+.py <==
# -*- coding: utf8 -*-
'''
DB MODULE - ODBC for TSQL
*******************************
***  DB MODULE - ODBC for TSQL
*******************************
$RCSfile: dbmod_odbc_tsql.py,v $
$Revision: 1302 $
$Author: j.holetschek $
$Date: 2013-07-08 14:14:06 +0200 (Mo, 08 Jul 2013) $

The biocase.wrapper database module to access MS SQL Server databases.
You will need the mxODBC module to connect to a MS SQL Server DB via ODBC.
'''

from biocase.wrapper.sql.base import sqlBase
from biocase.wrapper.dbmod.base_dbmod import DBBasemodule
from biocase.wrapper.errorclasses import DbModuleMissingError, ConnectionError

import logging
log = logging.getLogger("pywrapper.dbmod")


#############################################
#
# CUSTOMIZATION for SQL Server
#
# OVERLOAD BASE CLASSES AND OTHER METHODS
#
#############################################

# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'
sqlBase.templates['select-limit'] = 'SELECT DISTINCT TOP #3 #1 FROM #2'
sqlBase.templates['count-distinct'] = 'COUNT(DISTINCT #1)'

# Works only with SQL Server 2012 (or later)
sqlBase.templates['select-limit-offset'] = 'SELECT DISTINCT #1 FROM #2 OFFSET #4 ROWS FETCH NEXT #3 ROWS ONLY'

# ESCAPE CHARACTERS
# -----------------
# characters that need to be escaped in SQL
# default list contains single quotes to be escaped into two single quotes
# but that's already done b< ceODBC
sqlBase.escape = {}


class DBmodule(DBBasemodule):
    __version__ = ''
    __dbmod__ = u'MS SQL Server 2012+ module using '

    def __init__(self, psfObj):
        # setting the psfObj reference to self.psfObj
        # call createDBconnection method
        DBBasemodule.__init__(self, psfObj)
        # dont use parameters but integrate all parameters into SQL strings:
        self.trans.integrateParametersInSQLString()

    # ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
    def createDBconnection(self):
        try:
            import pyodbc as odbc
            self.__class__.__dbmod__ += "pyODBC"
        except:
            try:
                import ceODBC as odbc
                self.__class__.__dbmod__ += "ceODBC"
            except:
                raise DbModuleMissingError('ceODBC/pyODBC')

        self.__class__.__version__ = odbc.version
        loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.database, self.psfObj.user, self.psfObj.password)
        try:
            return odbc.connect(loginParas)
        except:
            raise ConnectionError()

    def getTables(self):
        tables = []
        obj = self.conn if self.__dbmod__.endswith('ceODBC') else self.conn.cursor()
        for t in obj.tables():
            if (str(t[3]) == 'TABLE' or str(t[3]) == 'VIEW') and not (t[1] in ["INFORMATION_SCHEMA", "sys"] or t[2] in ["dtproperties", "sysconstraints", "syssegments"]):
                tables.append(unicode(t[2]))
        return tables

    def getColumns(self, _table):
        try:
            obj = self.conn if self.__dbmod__.endswith('ceODBC') else self.conn.cursor()
            ret = [unicode(c[3]) for c in obj.columns(catalog=None, schema=None, table=_table)]
            if ret == None:
                return []
            return ret
        except:
            log.error("error fetching columns")
            return []

    def hasMetaInfo(self):
        return True
